(function(){
//////////////////////////////////////////////////////////////////////////////
const payload = {
    N: "{{ manjusaka-js-payload-N }}",
    nonce: "{{ manjusaka-js-payload-nonce }}",
    hints: "{{ manjusaka-js-payload-hints }}",
    content: "{{ manjusaka-js-payload-content }}",
};

const buffer = require("buffer");
const nonce = new buffer.Buffer(payload.nonce, "base64");
const generateQuestionsBasedKey = require("./questions-based-key.js");
const generatePasswordBasedKey = require("./password-based-key.js");
const openpgp = require("./openpgp.min.js");


function isString(x){
    return Object.prototype.toString.call(x) === "[object String]"
}

const usePassword = Boolean(
    isString(payload.hints) ^ Object.keys(payload.hints).length == 0);
const useQuestions = (
    isString(payload.hints) || Object.keys(payload.hints).length > 0)


var onProgress = function(){};

async function reveal(passwordPromise, questionsPromise){
    var passwordBasedKey = "";
    var questionsBasedKey = "";
    var passwordValidated = false;

    while(true){

        if(usePassword && !passwordValidated){
            passwordBasedKey = await generatePasswordBasedKey(
                nonce,
                (await passwordPromise()),
                onProgress
            );
        }
            
        // shall question hints be decrypted?
        var questions = {};
        if(usePassword && useQuestions){
            try{
                questions = JSON.parse(
                    (await openpgp.decrypt({
                        message: (
                            await openpgp.message.readArmored(payload.hints)),
                        passwords: [passwordBasedKey],
                    })).data
                );
                passwordValidated = true;
            } catch(e){
                passwordValidated = false;
                console.error(e);
                continue; // decryption failure. password error? do again. 
            }
        } else {
            questions = payload.hints;
        }

        // get the questions based key
        if(useQuestions){
            const answers = await questionsPromise(questions);
            questionsBasedKey = await generateQuestionsBasedKey(
                nonce,
                answers, // { question1: { answer1: "" }, ... }
                onProgress
            );
        }

        // decrypt main content
        const contentEncryptionKey = questionsBasedKey + "|" + passwordBasedKey;
        try{
            const decryptedContent = (await openpgp.decrypt({
                passwords: [contentEncryptionKey],
                message: (await openpgp.message.readArmored(payload.content)),
            })).data;
            return decryptedContent;
        } catch(e){
            console.error(e);
            continue;
        }
    }
}

//----------------------------------------------------------------------------
// HTML manipulation codes

const $ = require("jquery");
function manjusaka(el){
    console.log("Manjusaka booted.");
    
    var password = null;
    if(window.location.hash) password = window.location.hash.slice(1);

    function passwordPromise(){
        if(password){
            const x = password;
            password = null;
            return x; 
        }
        $("#input-password").show().find("input").val("");
        return new Promise(function(resolve, reject){
            $("#submit-password").one("click", function(){
                resolve($("#input-password").find("input").val());
                $("#input-password").hide();
            });
        });
    }

    function questionsPromise(hints){
        $("#input-questions").show();
        $("#questions-list").empty();

        for(var questionName in hints){
            var newEntry = $("#input-question-template")
                .clone().removeAttr("id")
                .appendTo("#questions-list")
            ;
            newEntry.find('[name="question"]').text(questionName);
            newEntry.find('input')
                .attr("placeholder", hints[questionName] || "")
                .data("question", questionName)
            ;
        }

        return new Promise(function(resolve, reject){
            $("#submit-questions").one("click", function(){
                var answers = {};
                $("#questions-list").find("input").each(function(){
                    answers[$(this).data("question")] = {
                        answer: $(this).val(),
                    };
                });
                resolve(answers);
                $("#input-questions").hide();
            });
        });
    }

    reveal(passwordPromise, questionsPromise).then(function(data){
        $("#reveal").html(data);
    });
}

onProgress = function(p){
    if(p < 100){
        $("#wait").show().find('[name="progress"]').text(Number(p).toFixed(1));
    } else {
        $("#wait").hide();
    }
}


$(function(){
    $("#noscript").hide();
    manjusaka("#manjusaka");
});

//////////////////////////////////////////////////////////////////////////////
})();
